import inspect
import sys
import konira
from cStringIO import StringIO
from konira    import output


describe "writer and ansi codes":


    before each:
        self.stdout = StringIO()
        self.writer = output.Writer(stdout=self.stdout)

    it "knows if it is atty or not":
        assert self.writer.isatty == False


    it "returns empty strings for colors if it is not atty":
        self.writer.isatty = False
        assert self.writer.color("") == ''


    it "returns empty strings for colors when for is None":
        self.writer.isatty = False
        assert self.writer.color(None) == ''


    it "raises a keyerror when you try to get an invalid color":
        self.writer.isatty = True
        raises KeyError: self.writer.color('foo')


    it "returns blue ansi codes":
        self.writer.isatty = True
        assert self.writer.color('blue') == '\033[94m'
        

    it "returns green ansi codes":
        self.writer.isatty = True
        assert self.writer.color('green') == '\033[92m'


    it "returns yellow ansi codes":
        self.writer.isatty = True
        assert self.writer.color('yellow') == '\033[93m'


    it "returns red ansi codes":
        self.writer.isatty = True
        assert self.writer.color('red') == '\033[91m'


    it "returns bold ansi codes":
        self.writer.isatty = True
        assert self.writer.color('bold') == '\033[1m'


    it "returns ends ansi codes":
        self.writer.isatty = True
        assert self.writer.color('ends') == '\033[0m'



describe "writer stdout ouput":


    before each:
        self.stdout = StringIO()
        self.writer = output.Writer(stdout=self.stdout)


    it "println to stdout the string correctly without colors":
        self.writer.println("a string with no colors")
        assert self.stdout.getvalue() == "a string with no colors"


    it "writes to stdout without newlines but with colors":
        self.writer.isatty = True
        self.writer.write("A blue string", "blue")
        assert self.stdout.getvalue() == '\033[94mA blue string\033[0m'


    it "writes to stdout without newlines with no colors if not atty":
        self.writer.write("A blue string", "blue")
        assert self.stdout.getvalue() == 'A blue string'


    it "writelns with newline and colors":
        self.writer.isatty = True
        self.writer.writeln("A blue string", "blue")
        assert self.stdout.getvalue() == '\n\033[94mA blue string\033[0m'


    it "writelns with newline and no colors":
        self.writer.writeln("A blue string", "blue")
        assert self.stdout.getvalue() == '\nA blue string'


    it "writes n number of newlines":
        self.writer.newline(lines=3)
        assert self.stdout.getvalue() == '\n\n\n'


    it "writes a single newline by default":
        self.writer.newline()
        assert self.stdout.getvalue() == '\n'


    it "returns valid green strings on demand":
        self.writer.isatty = True
        assert self.writer.green("A green string") == '\033[92mA green string\033[0m'


    it "returns valid red strings on demand":
        self.writer.isatty = True
        assert self.writer.red("A red string") == '\033[91mA red string\033[0m' 


    it "returns valid bold strings on demand":
        self.writer.isatty = True
        assert self.writer.bold("A bold string") == '\033[1mA bold string\033[0m' 



describe "terminal writer stdout ouput":


    before each:
        self.stdout         = StringIO()
        self.writer         = output.Writer(stdout=self.stdout)
        self.twriter        = output.TerminalWriter(False)
        self.twriter.writer = self.writer

    after each:
        self.stdout = StringIO()
        
    it "outputs green spec titles":
        self.twriter.green_spec("green spec title")
        assert self.stdout.getvalue() == "\n    Green spec title"


    it "outputs a dot for green specs when dotted option is passed":
        self.twriter.dotted = True
        self.twriter.green_spec("green spec title")
        assert self.stdout.getvalue() == '.'


    it "outputs red spec titles":
        self.twriter.red_spec("red spec title")
        assert self.stdout.getvalue() == "\n    Red spec title"


    it "outputs an eff for red specs when dotted option is passed":
        self.twriter.dotted = True
        self.twriter.red_spec("red spec title")
        assert self.stdout.getvalue() == 'F'


    it "is None when dotted for out case":
        self.twriter.dotted = True
        assert self.twriter.out_case("an out case") == None
        assert self.stdout.getvalue() == ''


    it "outputs an out case when not dotted":
        assert self.twriter.dotted ==  False
        self.twriter.out_case("an out case") 
        assert self.stdout.getvalue() == '\n\nAn out case'



describe "footer output":


    before each:
        self.stdout = StringIO()
        self.writer = output.Writer(stdout=self.stdout)
        self.footer = output.out_footer


    it "outputs all passed when no failures and cases are more than zero":
        self.footer(1, 0, 0, std=self.writer)
        assert self.stdout.getvalue() == "\n\n\nAll 1 spec passed in 0 secs.\n"


    it "outputs plural specs when no failures and cases are more than zero":
        self.footer(2, 0, 0, std=self.writer)
        assert self.stdout.getvalue() == "\n\n\nAll 2 specs passed in 0 secs.\n"


    it "outputs failures as singular":
        self.footer(1, 1, 0, std=self.writer)
        assert self.stdout.getvalue() == "\n\n\n1 spec failed, 1 total in 0 secs.\n"



describe "report results":


    before all:
        try:
            assert False
        except Exception, e:
            trace = inspect.trace()
            self.false_failure = dict(
                            failure  = sys.exc_info(),
                            trace    = trace,
                            exc_name = e.__class__.__name__
                           ) 
            #self.false_trace = inspect.trace()[0]


    before each:
        class FakeObject(object): pass
        self.stdout         = StringIO()
        self.writer         = output.Writer(stdout=self.stdout)
        self.results        = FakeObject()
        self.results.config = {}
        self.reporter       = output.ReportResults


    it "writes a nice footer":
        self.results.total_cases    = 1
        self.results.total_failures = 1
        self.results.elapsed        = 0
        result = self.reporter(self.results, self.writer)
        result.footer()
        assert self.stdout.getvalue() == "\n\n\n1 spec failed, 1 total in 0 secs.\n"


    it "when sorting it returns only ten items":
        profiles = [(a, a, a) for a in range(0,15)]
        self.results.profiles = profiles 
        result = self.reporter(self.results, self.writer)
        assert len(result._sort_profiles()) == 10
        

    it "when sorting it does so in descending order":
        profiles = [(a, a, a) for a in range(0,15)]
        self.results.profiles = profiles 
        result = self.reporter(self.results, self.writer)
        assert result._sort_profiles()[0] == (14,14,14)
        assert result._sort_profiles()[1] == (13,13,13)
        assert result._sort_profiles()[2] == (12,12,12)


    it "outputs normalized test case names in the profiler":
        profiles = [(a, 'test_case', a) for a in range(0,15)]
        self.results.profiles = profiles 
        result = self.reporter(self.results, self.writer)
        result.profiler()
        assert "14 - Test case" in self.stdout.getvalue()


    it "displays a header showgin profiling is enabled":
        profiles = [(a, 'test_case', a) for a in range(0,15)]
        self.results.profiles = profiles 
        result = self.reporter(self.results, self.writer)
        result.profiler()
        expected = "Profiling enabled\n10 slowest tests shown:\n" 
        assert expected in self.stdout.getvalue()


    it "orders single profile messages by time and case name":
        result = self.reporter(self.results, self.writer)
        result._output_profiles((0, 'test_case'))
        assert self.stdout.getvalue() == '\n0 - Test case'


    it "only gices the 10 first digits in elapsed time":
        result = self.reporter(self.results, self.writer)
        result._output_profiles(('0.00000099999', 'test_case'))
        assert self.stdout.getvalue() == '\n0.00000099 - Test case'


    it "can report profiling errors and failures all at once":
        self.results.total_cases    = 1
        self.results.total_failures = 1
        self.results.elapsed        = 0
        self.results.failures = [self.false_failure]
        self.results.errors = [self.false_failure]
        self.results.profiles = [('0', 'Test_case')]
        result = self.reporter(self.results, self.writer)
        result.report()
        output = self.stdout.getvalue()
        assert 'AssertionError'        in output
        assert 'Failures'              in output
        assert 'Starts and Ends'       in output
        assert 'File:'                 in output
        assert 'case_output.py'        in output
        assert '1 spec failed, 1 total in 0 secs.' in output







from cStringIO import StringIO
from konira    import tokenizer


describe "quote remover":


    before each:
        self.remover = tokenizer.quote_remover


    it "removes commas from strings":
        assert self.remover("a,separated,string") == "aseparatedstring"


    it "removes commas respecting whitespace":
        assert self.remover("a, separated, string") == "a separated string"


    it "removes single quotes from strings":
        assert self.remover("a ' string '") == "a  string "


    it "removes a combination of commas and quotes":
        assert self.remover("a, 'string',") == "a string"


    it "removes dots from strings":
        assert self.remover("a. dotted. string.") == "a dotted string"


    it "respects camel case strings":
        assert self.remover("a Camel Case string") == "a Camel Case string"



describe "valid method names":


    before each:
        self.valid = tokenizer.valid_method_name


    it "replaces whitespace with underscores":
        assert self.valid("a name with underscores") == "it_a_name_with_underscores"


    it "removes double quotes from strings":
        assert self.valid('a double \"quoted\" string') == "it_a_double_quoted_string"


    it "removes commas from strings":
        assert self.valid('a ,in a string,') == "it_a_in_a_string"


    it "removes commas quotes and double quotes from strings":
        assert self.valid('a \", in a string\'') == 'it_a__in_a_string'


    it "removes dots from strings":
        assert self.valid("a. dotted. string.") == "it_a_dotted_string"


    it "respects camel case from strings":
        assert self.valid("a CamelCase string") == "it_a_CamelCase_string"



describe "valid class names":


    before each:
        self.valid = tokenizer.valid_class_name


    it "replaces whitespace with underscores":
        assert self.valid("a string with whitespace") == "Case_a_string_with_whitespace"


    it "removes double quotes from strings":
        assert self.valid("a string\"") == "Case_a_string"


    it "removes dots and commas from strings":
        assert self.valid("a. string, \'") == "Case_a_string_"


    it "removes single quotes from strings":
        assert self.valid("\'single quote") == "Case_single_quote"


    it "respects camel case in strings":
        assert self.valid("my Test") == "Case_my_Test"



describe "valid raises detection":


    before each:
        self.valid = tokenizer.valid_raises


    it "is True when the value is None":
        assert self.valid(None) == True


    it "is True when the value is an empty string":
        assert self.valid(" ") == True


    it "is false when the value has a character at the start":
        assert self.valid("d ") == False


    it "is false when the value has a character at the end":
        assert self.valid("   d") == False


    it "is true when there are non whitespace chars":
        assert self.valid("\n ") == True


    it "is true when an empty string is passed":
        assert self.valid("") == True



describe "translate dsl into valid Python":
    

    before each:
        self.translate = tokenizer.translate
        self.line      = StringIO


    it "never do implicit imports":
        line = self.line('').readline
        assert self.translate(line) == [[0, '']]

    
    it "translates describe into a class":
        line = self.line('describe "my test class":\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'class']
        assert result[1]   == [1, 'Case_my_test_class']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'object']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']
        

    it "translates an it to a def":
        line = self.line('it "should test my method":\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'def']
        assert result[1]   == [3, 'it_should_test_my_method']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'self']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates a describe with inheritance":
        line = self.line('describe "my test class", Foo:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'class']
        assert result[1]   == [1, 'Case_my_test_class']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'Foo']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates a describe with inheritance regardless of space":
        line = self.line('describe "my test class",Foo:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'class']
        assert result[1]   == [1, 'Case_my_test_class']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'Foo']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates a describe with inheritance with a lot of whitespace":
        line = self.line('describe "my test class"    ,          Foo:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'class']
        assert result[1]   == [1, 'Case_my_test_class']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'Foo']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates skip if constructors":
        line = self.line('skip if:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'def']
        assert result[1]   == [1, '_skip_if']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'self']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates before all constructors":
        line = self.line('before all:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'def']
        assert result[1]   == [1, '_before_all']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'self']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates before each constructors":
        line = self.line('before each:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'def']
        assert result[1]   == [1, '_before_each']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'self']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates after each constructors":
        line = self.line('after each:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'def']
        assert result[1]   == [1, '_after_each']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'self']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates after all constructors":
        line = self.line('after all:\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'def']
        assert result[1]   == [1, '_after_all']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'self']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "translates raises into with statements":
        line = self.line('raises IOError: foo()')
        result = self.translate(line.readline)
        assert len(result) == 9
        assert result[0]   == [1, 'with konira.tools.raises']
        assert result[1]   == [51, '(']
        assert result[2]   == [1, 'IOError']
        assert result[3]   == [51, ')']
        assert result[4]   == [51, ':']
        assert result[5]   == [1, 'foo']
        assert result[6]   == [51, '(']
        assert result[7]   == [51, ')']
        assert result[8]   == [0, '']


    it "does not translate a regular class":
        line = self.line('class Foo(object):\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'class']
        assert result[1]   == [1, 'Foo']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'object']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']


    it "does not translate a regular method":
        line = self.line('def foo(self):\n pass')
        result = self.translate(line.readline)
        assert len(result) == 11
        assert result[0]   == [1, 'def']
        assert result[1]   == [1, 'foo']
        assert result[2]   == [51, '(']
        assert result[3]   == [1, 'self']
        assert result[4]   == [51, ')']
        assert result[5]   == [51, ':']
        assert result[6]   == [4, '\n']
        assert result[7]   == [5, ' ']
        assert result[8]   == [1, 'pass']
        assert result[9]   == [6, '']
        assert result[10]  == [0, '']




"""
Write and report coverage data with the 'coverage' package.
Original code by Ross Lawley. 

Requires Ned Batchelder's excellent coverage:
http://nedbatchelder.com/code/coverage/
"""
import sys


def import_coverage():
    try:
        from coverage import coverage
        return coverage()
    except ImportError:
        msg = "coverage is not installed or not available in current sys.path\n"
        msg +="make sure it is installed properly and try again."
        sys.stdout.write(msg)
        sys.exit(2)


class DoCoverage(object):


    __defaults__ = dict(
                    show_missing  = False,
                    report        = 'report',
                    directory     = 'coverage',
                    ignore_errors = True,
                    coverpackages = False
                    )


    def __init__(self, options=None):
        self.options = self.__defaults__
        if options:
            self.options.update(options)
        self._coverage = import_coverage()
        self._coverage.use_cache(False)
        self._coverage.start()


    def konira_terminal_summary(self):
        self._coverage.stop()
        self._coverage.save()

        show_missing = self.options.get('show_missing')
        report_type  = self.options.get('report')
        directory    = self.options.get('directory')
        ignore_err   = self.options.get('ignore_errors')
        
        report_args = {
            'ignore_errors': ignore_err,
        }

        coverpackages = self.options.get('coverpackages')
        if coverpackages:
            modules = report_args['morfs'] = []
            for name, module in sys.modules.items():
                if module is not None and hasattr(module, '__file__'):
                    fn = module.__file__
                    for pkg in coverpackages:
                        if name.startswith(pkg):
                            modules.append(fn)
                            break
        
        
        if report_type == 'report':
            self._coverage.report(show_missing=show_missing, file=sys.stdout,
                    **report_args)
        elif report_type == 'annotate':
            self._coverage.annotate(directory=directory, **report_type)
        elif report_type == 'html':
            self._coverage.html_report(directory=directory, **report_args)



import os
import konira
from cStringIO                  import StringIO
from konira.collector           import FileCollector, globals_from_file


describe "path collection":


    before each:
        self.f = FileCollector(path='/asdf')
    

    it "should be a list":
        assert isinstance(self.f, list)


    it "does not match normal python files":
        py_file = "foo.py"
        assert self.f.valid_module_name.match(py_file) == None


    it "matches upper case python cases":
        py_file = "CASE_foo.py"
        assert self.f.valid_module_name.match(py_file) 


    it "does not match case without underscores":
        py_file = "casfoo.py"
        assert self.f.valid_module_name.match(py_file) == None
        

    it "does not match if it doesn't start with case_":
        py_file = "foo_case.py"
        assert self.f.valid_module_name.match(py_file) == None


    it "matches if it has camelcase":
        py_file = "CaSe_foo.py"
        assert self.f.valid_module_name.match(py_file)


    it "does not match if it starts with underscore":
        py_file = "case_foo.py"
        assert self.f.valid_module_name.match(py_file)



describe 'global values from file':


    before all:
        with open('/tmp/case_test.py', 'w') as self.case_test:
            self.case_test.write("import sys")


    after all:
        try:
            os.remove('/tmp/case_test.py')
        except:
            pass # who cares if you can't


    it 'should see globals':
        globs = globals_from_file('/tmp/case_test.py')
        assert globs
        assert len(globs) == 2


    it "raises IOError when it tries an invalid path":
        raises IOError: globals_from_file('/foo/bar/foo.py')


    it "raises TypeError when no filename is passed":
        raises TypeError: globals_from_file()



from konira import argopts


describe "parsing arguments":


    it "removes the first item in the list always":
        parser = argopts.ArgOpts([])
        parser.parse_args(['foo'])
        assert parser.args == []


    it "matches an option in arguments":
        parser = argopts.ArgOpts(['--foo'])
        parser.parse_args(['/bin/konira', '--foo'])
        assert parser.args  == ['--foo']
        assert parser.match == ['--foo']


    it "matches arguments with no values":
        parser = argopts.ArgOpts(['--foo'])
        parser.parse_args(['/bin/konira', '--foo'])
        assert parser._arg_count['--foo'] == 0
        assert parser._count_arg[0]       == '--foo'


    it "matches arguments with values":
        parser = argopts.ArgOpts(['--foo'])
        parser.parse_args(['/bin/konira', '--foo', 'BAR'])
        assert parser.match               == ['--foo']
        assert parser._arg_count['--foo'] == 0
        assert parser._count_arg[1]       == 'BAR'


    it "matches valid configured options only":
        parser = argopts.ArgOpts(['--fuuu'])
        parser.parse_args(['/bin/konira', '--foo', '--meh'])
        assert parser.match == []

    
    it "matches mixed values and arguments":
        parser = argopts.ArgOpts(['--foo', '--bar'])
        parser.parse_args(['/bin/konira', '--foo', 'FOO', '--bar'])
        assert parser.match == ['--foo', '--bar']
        assert parser._arg_count['--foo'] == 0
        assert parser._arg_count['--bar'] == 2
        assert parser._count_arg[1]       == 'FOO'
        assert parser._count_arg.get(3)   == None



describe "get values from arguments":


    before each:
        self.parser = argopts.ArgOpts(['--foo'])
        self.parser.parse_args(['/bin/konira', '--foo', 'BAR', '--bar'])


    it "returns a valid value from a matching argument":
        assert self.parser.get_value('--foo') == 'BAR'


    it "returns None when an argument does not exist":
        assert self.parser.get_value('--meh') == None


    it "returns None when an argument does not have a value":
        assert self.parser.get_value('--bar') == None



describe "has or does not have options":


    before each:
        self.parser = argopts.ArgOpts(['--foo'])
        self.parser.parse_args(['/bin/konira', '--foo', 'BAR', '--bar'])


    it "accepts lists and returns if one matches":
        opt = ['a', 'b', '--foo']
        assert self.parser.has(opt) == True


    it "returns none if cannot match from a list":
        opt = ['a', 'b']
        assert self.parser.has(opt) == False


    it "deals with single items that match":
        assert self.parser.has('--foo') == True


    it "returns False when a single item does not match":
        assert self.parser.has('--asdadfoo') == False



describe "catches help":


    before each:
        self.parser = argopts.ArgOpts(['--foo'])


    it "catches only help if it sees it as an argument":
        self.parser.args = ['foo', 'bar']
        assert self.parser.catches_help() == False


    it "catches single dash h":
        self.parser.args = ['-h']
        assert self.parser.catches_help() == True


    it "catches double dash h":
        self.parser.args = ['--h']
        assert self.parser.catches_help() == True


    it "catches double dash help":
        self.parser.args = ['--help']
        assert self.parser.catches_help() == True


    it "catches help if it sees it as an argument":
        self.parser.args = ['-h']
        assert self.parser.catches_help() == True



describe "catches version":


    before each:
        self.parser = argopts.ArgOpts(['--foo'])


    it "catches only version if it sees it as an argument":
        self.parser.args = ['foo', 'bar']
        assert self.parser.catches_version() == False


    it "catches double dash version":
        self.parser.args = ['foo', '--version']
        assert self.parser.catches_version() == True


    it "catches version if it sees it as an argument":
        self.parser.args = ['foo', 'version']
        assert self.parser.catches_version() == True




from konira import util


describe "name convertion":


    it "replaces underscore with whitespace":
        name = "a_white_space_string"
        assert util.name_convertion(name) == 'A white space string'


    it "removes whitespaces from outer string":
        name = " a_white_space_string "
        assert util.name_convertion(name) == 'a white space string'


    it "removes case from a name string":
        name = "Case_foo_bar"
        assert util.name_convertion(name) == 'foo bar'


    it "does not remove case if not uppercase":
        name = "case_foo_bar"
        assert util.name_convertion(name) == 'Case foo bar'



describe "get class name":


    it "returns itself when it str does not start with case":
        name = "a string"
        assert util.get_class_name(name) == name 


    it "returns none when does not start with Case":
        name = """<class '_stdout_messages_for_the_command_line'>"""
        assert util.get_class_name(name) == None
        

    it "parses correctly when a valid class name is sent":
        name = """<class 'Case_stdout_messages_for_the_command_line'>"""
        assert util.get_class_name(name) == 'Case_stdout_messages_for_the_command_line'



describe "stop watch":


    it "returns no more than 5 chars":
        swatch = util.StopWatch()
        assert len(swatch.elapsed()) == 5




import konira
from konira.exc import KoniraReassertError
from konira     import tools


describe "raises tool":

    before each:
        self.r = tools.raises

    it "is True when the exception matches":
        with self.r(): raise



describe "assert raises":

    it "raises when the exception does not match":
        assert_raises = tools.AssertRaises(Exception, None)
        raises KoniraReassertError: assert_raises.__exit__(None, None, None)


    it "is true when the exception matches":
        assert_raises = tools.AssertRaises(Exception, None)
        assert assert_raises.__exit__(Exception, None, None)


    it "raises when the exception message does not match":
        assert_raises = tools.AssertRaises(Exception, "foo")
        raises KoniraReassertError: assert_raises.__exit__(Exception, None, None)







class ArgOpts(object):


    def __init__(self, options):
        self.options    = options
        self.help       = ['-h', '--h', '--help', 'help']
        self.version    = ['--version', 'version']
        self._arg_count = {}
        self._count_arg = {}
        

    def parse_args(self, argv):
        self.args = argv[1:]
        self.match = [i for i in argv if i in self.options]
        for count, argument in enumerate(self.args):
            self._arg_count[argument] = count
            self._count_arg[count]    = argument


    def get_value(self, opt):
        count = self._arg_count.get(opt)
        if count == None:
            return None
        value = self._count_arg.get(count+1)
        return value


    def has(self, opt):
        if type(opt) == list:
            for i in opt:
                if i in self._arg_count.keys():
                    return True
            return False
        if opt in self._arg_count.keys():
            return True
        return False


    def catches_help(self):
        if [i for i in self.args if i in self.help]:
            return True
        return False


    def catches_version(self):
        if [i for i in self.args if i in self.version]:
            return True
        return False




import os
import sys
from cStringIO import StringIO
from konira    import KoniraCommands



describe "stdout messages for the command line":


    before all:
        self.commands = KoniraCommands(argv=[], parse=False, test=True)


    before each:
        self.original_stdout = sys.stdout
        self.original_stderr = sys.stderr
        sys.stderr = self.mystderr = StringIO()
        sys.stdout = self.mystdout = StringIO()


    after each:
        sys.stdout = self.original_stdout
        sys.stderr = self.original_stderr

    
    it "should go to stdout if a flag is passed in":
        self.commands.msg("A stdout message")
        assert self.mystdout.getvalue() == "A stdout message\n"


    it "does not output stderr messages when stdout flaf is set":
        self.commands.msg("A stdout message")
        assert self.mystderr.getvalue() == ""


    it "outputs to stderr if a flag is passed in":
        self.commands.msg("A stderr message", stdout=False)
        assert self.mystderr.getvalue() == "A stderr message\n"


    it "does not output to stdout if the stderr flag is set":
        self.commands.msg("A stderr message", stdout=False)
        assert self.mystdout.getvalue() == ""



describe "getting tests and paths from arguments":


    before all:
        self.commands = KoniraCommands(argv=[], parse=False, test=True)


    it "returns the path only when it cannot split further":
        path = "/path/to/my/tests"
        from_path = self.commands.test_from_path(path)
        assert from_path.get('path') == path
        assert len(from_path)        == 1


    it "returns a class name if it can split more than once":
        path = "/path/to/my/tests::myclass"
        from_path = self.commands.test_from_path(path)
        assert len(from_path)              == 2
        assert from_path.get('class_name') == 'Case_myclass'
        assert from_path.get('path')       == '/path/to/my/tests'


    it "returns a valid case class name if it can split the path":
        path = "/path/to/my/tests::my separated class"
        from_path = self.commands.test_from_path(path)
        assert len(from_path)              == 2
        assert from_path.get('class_name') == 'Case_my_separated_class'
        assert from_path.get('path')       == '/path/to/my/tests'


    it "returns the path class and method name if it can":
        path = "/path::my class::my method"
        from_path = self.commands.test_from_path(path)
        assert len(from_path)               == 3
        assert from_path.get('class_name')  == 'Case_my_class'
        assert from_path.get('method_name') == 'it_my_method'
        assert from_path.get('path')        == '/path'


    it "always return the path if it fails to split garbage":
        path = "/path$$$"
        from_path = self.commands.test_from_path(path)
        assert len(from_path)        == 1
        assert from_path.get('path') == path



describe "getting parsed paths from arguments":


    before all:
        self.commands = KoniraCommands(argv=[], parse=False, test=True)
        self.cwd = os.getcwd()


    it "returns the current only the cwd for invalid paths":
        args = ['/bin/konira', '/a/completely/wrong/path']
        path_dict = self.commands.path_from_argument(args)
        assert len(path_dict)        == 1
        assert path_dict.get('path') == self.cwd


    it "if it is a valid path then return a dict with other options":
        args = ['/bin/konira', '/tmp']
        path_dict = self.commands.path_from_argument(args)
        assert len(path_dict)               == 3
        assert path_dict.get('class_name')  == None
        assert path_dict.get('method_name') == None
        assert path_dict.get('path')        == '/tmp'


    it "returns classes with paths if it is valid":
        args = ['/bin/konira', '/tmp::my class']
        path_dict = self.commands.path_from_argument(args)
        assert len(path_dict)               == 3
        assert path_dict.get('class_name')  == 'Case_my_class'
        assert path_dict.get('method_name') == None
        assert path_dict.get('path')        == '/tmp'

        
    it "returns classes and method when valid paths are passed":
        args = ['/bin/konira', '/tmp::my class:: my method']
        path_dict = self.commands.path_from_argument(args)
        assert len(path_dict)               == 3
        assert path_dict.get('class_name')  == 'Case_my_class'
        assert path_dict.get('method_name') == 'it_my_method'
        assert path_dict.get('path')        == '/tmp'


    it "returns the cwd if no paths are sent":
        args = ['/bin/konira']
        path_dict = self.commands.path_from_argument(args)
        assert len(path_dict)        == 1
        assert path_dict.get('path') == self.cwd



import os
import inspect
from konira        import Runner
from konira.runner import TestEnviron


describe "safe test environment calls":

    before all:
        def _raise():
            assert False
        
        def _not_raise():
            assert True
        self._raise     = _raise
        self._not_raise = _not_raise

    before each:
        self.runner = Runner(None, {})


    it "appends an error when an exception happens":
        self.runner.safe_environ_call(self._raise)
        assert len(self.runner.errors)               == 1
        assert self.runner.errors[0].get('exc_name') == 'AssertionError'


    it "does not append errors if the call does not raise":
        self.runner.safe_environ_call(self._not_raise)
        assert len(self.runner.errors) == 0
        assert self.runner.errors      == []


describe "save skip calls":


    before all:
        def _raise():
            assert False
        
        def _not_raise():
            assert True

        def _skip_raise():
            raise KoniraNoSkip

        self._raise      = _raise
        self._not_raise  = _not_raise
        self._skip_raise = _skip_raise


    before each:
        self.runner = Runner(None, {})


    it "returns true when it does not raise":
        result = self.runner.safe_skip_call(self._not_raise)
        assert result


    it "returns false when it raises anything but a NoSkip exception":
        result = self.runner.safe_skip_call(self._raise)
        assert result == False


    it "returns false when it raises a KoniraNoSkip exception":
        result = self.runner.safe_skip_call(self._skip_raise)
        assert result == False


describe "running a spec suite":


    before all:
        self.cwdir    = os.getcwd()
        self.cwfile   = inspect.getfile(inspect.currentframe())


    before each:
        self.runner = Runner(None, {})


    it "appends an error when it raises at class collection":
        self.runner.paths = [self.cwdir]
        self.runner.run()
        assert len(self.runner.errors) == 1
        assert self.runner.errors[0].get('exc_name') == 'IOError'



describe "get test environ setup values":


    before each:
        class Object(object):
            pass
        self.object = Object


    it "does not have any environ attribute":
        environ = TestEnviron(self.object)
        assert environ.has_skip_if     == False
        assert environ.has_before_all  == False
        assert environ.has_before_each == False
        assert environ.has_after_all   == False
        assert environ.has_after_each  == False


    it "checks for skip if properties":
        skip_if          = self.object()
        skip_if._skip_if = True
        environ          = TestEnviron(skip_if)
        assert environ.has_skip_if     == True
        assert environ.has_before_all  == False
        assert environ.has_before_each == False
        assert environ.has_after_all   == False
        assert environ.has_after_each  == False


    it "checks for before all properties":
        before_all = self.object()
        before_all._before_all = True
        environ = TestEnviron(before_all)
        assert environ.has_skip_if     == False
        assert environ.has_before_all  == True
        assert environ.has_before_each == False
        assert environ.has_after_all   == False
        assert environ.has_after_each  == False


    it "checks for before each properties":
        before_each = self.object()
        before_each._before_each = True
        environ = TestEnviron(before_each)
        assert environ.has_skip_if     == False
        assert environ.has_before_all  == False
        assert environ.has_before_each == True
        assert environ.has_after_all   == False
        assert environ.has_after_each  == False


    it "checks for after all properties":
        after_all = self.object()
        after_all._after_all = True
        environ = TestEnviron(after_all)
        assert environ.has_skip_if     == False
        assert environ.has_before_all  == False
        assert environ.has_before_each == False
        assert environ.has_after_all   == True
        assert environ.has_after_each  == False


    it "checks for after each properties":
        after_each = self.object()
        after_each._after_each = True
        environ = TestEnviron(after_each)
        assert environ.has_skip_if     == False
        assert environ.has_before_all  == False
        assert environ.has_before_each == False
        assert environ.has_after_all   == False
        assert environ.has_after_each  == True


    it "catches all properties set":
        properties = self.object()
        properties._after_all   = True
        properties._after_each  = True
        properties._before_all  = True
        properties._before_each = True
        properties._skip_if     = True

        environ = TestEnviron(properties)
        assert environ.has_skip_if     == True
        assert environ.has_before_all  == True
        assert environ.has_before_each == True
        assert environ.has_after_all   == True
        assert environ.has_after_each  == True





import sys
import traceback
from os.path                import dirname, abspath
from konira.util            import name_convertion 
from konira.exc             import konira_assert


class ReportResults(object):


    def __init__(self, results, writer=None):
        self.results = results
        self.config  = self.results.config
        self.writer  = writer
        if not self.writer:
            self.writer  = Writer()


    def report(self):
        self.writer.newline()
        if self.config.get('profiling'):
            self.profiler()
        if self.results.failures:
            self.failures()
        if self.results.errors:
            self.errors()
        self.footer()


    def failures(self):
        format_exc = ExcFormatter(self.results.failures, self.config, self.writer)
        format_exc.output_failures()


    def errors(self):
        format_exc = ExcFormatter(self.results.errors, self.config, self.writer)
        format_exc.output_errors()


    def footer(self):
        out_footer(self.results.total_cases, 
                self.results.total_failures, 
                self.results.elapsed,
                self.writer)


    def profiler(self):
        self.writer.newline()
        prof_message = "Profiling enabled\n10 slowest tests shown:\n"
        self.writer.write(prof_message, 'green')
        for p in self._sort_profiles():
            self._output_profiles(p)


    def _sort_profiles(self):
        _sorted =  sorted(self.results.profiles, 
                          key=lambda p: p[0], 
                          reverse=True)
        return _sorted[:10]


    def _output_profiles(self, data):
        str_time = str(data[0])[:10]
        elapsed  = self.writer.bold(str_time)
        case     = self.writer.green(name_convertion(data[1]))
        message  = "%s - %s" % (elapsed, case)
        self.writer.writeln(message)



class TerminalWriter(object):


    def __init__(self, dotted):
        self.dotted = dotted
        self.writer = Writer()


    def green_spec(self, title):
        string = "    %s" % (name_convertion(title))
        if not self.dotted:
            self.writer.writeln(string, 'green')
        else:
            self.writer.println('.')
            self.writer.flush()


    def red_spec(self, title):
        string = "    %s" % (name_convertion(title))
        if not self.dotted:
            self.writer.writeln(string, 'red')
        else:
            self.writer.println('F')
            self.writer.flush()


    def out_case(self, title):
        if not self.dotted:
            self.writer.println("\n\n%s" % name_convertion(title))


    def out_bold(self, string):
        self.writer.write(string, 'bold')


    def skipping(self):
        if not self.dotted:
            self.write.println(' ...skipping')
        else:
            self.writer.println('S')
            self.writer.flush()



def out_footer(cases, failures, elapsed, std=None):
    if not std:
        std = Writer()
    std.newline(2)
    if not failures and cases > 0:
        spec_verb = 'specs' if cases > 1 else 'spec'
        string = "All %s %s passed in %s secs.\n" % (cases, spec_verb, elapsed)
        std.writeln(string, 'green')
        return
    elif failures:
        spec_verb = 'specs' if failures > 1 else 'spec'
        string = "%s %s failed, %s total in %s secs.\n" % (failures, spec_verb, cases, elapsed)
        std.writeln(string, 'red')
        return
    if not cases or cases == 0:
        string = "No cases/specs collected.\n"
        std.writeln(string, 'bold')



def format_file_line(filename, line):
    return Writer().bold("%s:%s:" % (filename, line))



class ExcFormatter(object):


    def __init__(self, failures, config, std=None):
        self.config      = config
        self.failures    = failures
        self.failed_test = 1
        self.std = std
        if not self.std:
            self.std = Writer()


    def output_failures(self):
        self.std.writeln('Failures:\n---------', 'red')
        for failure in self.failures:
            self.single_exception(failure)


    def output_errors(self):
        self.std.writeln('\n\nErrors:\n-------', 'red')
        for error in self.failures:
            error = self.build_error_output(error)
            self.failure_header(error['description'])
            self.std.writeln("File: ", 'red')
            self.std.println(format_file_line(error['filename'], error['lineno']))
            if self.config.get('traceback') and error['text']:
                self.std.writeln(error['text'], 'red')


    def build_error_output(self, error):
        exc = {}
        p_error = PrettyExc(error['failure'], error=True)
        exc['description'] = p_error.exception_description
        try: # See if we have info at the message level and use that
            exc['filename'] = p_error.exc_value.filename
            exc['lineno']   = p_error.exc_value.lineno
        except:
            exc['filename']    = p_error.exception_file
            exc['lineno']      = p_error.exception_line
        exc['text']        = p_error.formatted_exception
        return exc


    def single_exception(self, failure):
        exc        = failure.get('failure')
        name       = failure.get('exc_name')
        trace      = failure.get('trace')
        pretty_exc = PrettyExc(exc)

        self.failure_header(pretty_exc.exception_description)
        starts = pretty_exc.exception_file_start
        ends   = pretty_exc.exception_file_end

        if starts == ends:
            self.std.writeln("Starts and Ends: ", 'red')
            self.std.println(format_file_line(pretty_exc.exception_file_start, pretty_exc.exception_line_start))
        else:
            self.std.writeln("Starts: ", 'red')
            self.std.println(format_file_line(pretty_exc.exception_file_start, pretty_exc.exception_line_start))
            self.std.writeln("Ends: ", 'red')
            self.std.println(format_file_line(pretty_exc.exception_file_end, pretty_exc.exception_line_end))

        if self.config.get('traceback'):
            if name == 'AssertionError':
                reassert = konira_assert(trace)            
                if reassert:
                    self.assertion_diff(reassert)
                    self.std.writeln(pretty_exc.formatted_exception)
                else:
                    self.std.writeln(pretty_exc.formatted_exception)
            else:
                self.std.writeln(pretty_exc.formatted_exception)


    def assertion_diff(self, diff):
        self.std.writeln("Assert Diff: %s" % str(diff[0].__repr__()), 'red') 

        # remove actual assert line
        diff.pop(0)
        for line in diff:
            if "?" and "^" in line:
                self.std.writeln(self.std.red('E            '+line))
            else:
                self.std.writeln(self.std.red('E            ')+line)


    def failure_header(self, name):
        string = "\n%s ==> %s" % (self.failed_test, name)
        self.failed_test += 1
        self.std.writeln(string, 'red')



class PrettyExc(object):


    def __init__(self, exc_info, error=False):
        self.error = error
        self.exc_type, self.exc_value, self.exc_traceback = exc_info
        if self.error:
            self.exc_traceback =  self._last_traceback(self.exc_traceback)
        self.exc_traceback        = self._remove_konira_from_traceback(self.exc_traceback)
        self.end_traceback        = self._last_traceback(self.exc_traceback)
        self.exception_file_start = self.exc_traceback.tb_frame.f_code.co_filename
        self.exception_line_start = self.exc_traceback.tb_lineno
        self.exception_file_end   = self.end_traceback.tb_frame.f_code.co_filename
        self.exception_line_end   = self.end_traceback.tb_lineno
        self.exception_line       = self.exc_traceback.tb_lineno
        self.exception_file       = self.exc_traceback.tb_frame.f_code.co_filename
        self.exc_info             = exc_info


    @property
    def formatted_exception(self):
        traceback_lines = traceback.format_exception(self.exc_type,
                                                     self.exc_value,
                                                     self.exc_traceback)
        return ''.join(traceback_lines)


    @property
    def indented_traceback(self):
        trace = self.formatted_exception.split('\n')
        add_indent = ["    "+i for i in trace]
        return '\n'.join(add_indent)


    def _remove_konira_from_traceback(self, traceback):
        if self.error: return traceback
        konira_dir = dirname(abspath(__file__))

        while True:
            try:
                frame    = traceback.tb_frame
                code     = frame.f_code
                filename = code.co_filename
                code_dir = dirname(abspath(filename))
                if code_dir != konira_dir:
                    break
                else:
                    traceback = traceback.tb_next
            except AttributeError:
                return traceback

        return traceback


    @property
    def exception_description(self):
        desc = traceback.format_exception_only(self.exc_type, self.exc_value)
        return self._short_exception_description(desc)


    def _short_exception_description(self, exception_description_lines):
        return exception_description_lines[-1].strip()


    def _last_traceback(self, tb):
        if tb is None: return tb
        while tb.tb_next:
            tb = tb.tb_next
        return tb



class Writer(object):


    def __init__(self, stdout=None):
        if not stdout:
            self.stdout = sys.__stdout__
        else:
            self.stdout = stdout
        self.out    = self.stdout.write
        self.isatty = self.stdout.isatty()
        self.flush  = self.stdout.flush


    def color(self, form):
        if not self.isatty: return ''
        if form == None: return ''
        available = dict(
                blue   = '\033[94m',
                green  = '\033[92m',
                yellow = '\033[93m',
                red    = '\033[91m',
                bold   = '\033[1m',
                ends   = '\033[0m'
            )
        try:
            return available[form]
        except:
            raise KeyError('%s is not a valid format/color' % form) 


    def println(self, string):
        self.out("%s" % string)


    def write(self, string, form):
        """No new line before or after"""
        color   = self.color(form)
        ends    = self.color('ends')
        out_str = "%s%s%s" % (color, string, ends)
        self.out(out_str)


    def writeln(self, string, form=None):
        """With a new line before and after"""
        color   = self.color(form)
        ends    = self.color('ends')
        out_str = "\n%s%s%s" % (color, string, ends)
        self.out(out_str)


    def newline(self, lines=1):
        nln = '\n'*lines
        self.out(nln)
        

    def green(self, string):
        """
        Makes incoming string output as green on the terminal
        """
        color   = self.color('green')
        ends    = self.color('ends')
        color_it = "%s%s%s" % (color, string, ends)
        return color_it


    def red(self, string):
        """
        Makes incoming string output as red on the terminal
        """
        color   = self.color('red')
        ends    = self.color('ends')
        color_it = "%s%s%s" % (color, string, ends)
        return color_it


    def bold(self, string):
        """
        Makes text bold in the terminal
        """
        color   = self.color('bold')
        ends    = self.color('ends')
        bold_it = "%s%s%s" % (color, string, ends)
        return bold_it




import sys
import os
from konira           import tokenizer
from cStringIO        import StringIO
from konira.collector import FileCollector
from konira.runner    import Runner
from konira.exc       import DontReadFromInput
from konira.util      import runner_options
from konira.ext       import cover
from konira.argopts   import ArgOpts
from konira.output    import ReportResults
import konira.tools

__version__ = '0.1.0'

class KoniraCommands(object):

    konira_help = """
konira: A test runner and DSL testing framework for writing readable, 
descriptive tests.

Version: %s

Run tests:
    konira [/path/to/cases] 
    konira ['/path/to/cases::case description::it description']

Control Options:
    --version, version  Shows the current installed version
    -s, no-capture      Avoids capturing stderr and stdout
    -x, fail            Stops at first fail
    -t, traceback       Shows tracebacks with errors/fails
    -d, dots            Displays '.' for passing and 'F' for failed tests.
    -p, profile         Enables profiling displaying the 10 slowest tests
                        forces dotted output.

Coverage Options:
    cover              Runs coverage and (optionally) includes information 
                       for selected packages
    --no-missing       Ommit missing lines in files

Matching Options:
    describe            Matches a case description (needs to be 
                        quoted)
    it                  Matches an 'it' spec (needs to be quoted)

    """ % __version__

    def __init__(self, argv=None, parse=True, test=False):
        self.test             = test
        self.config           = runner_options
        self.running_coverage = False
        self._original_stdout = sys.stdout
        self._original_stderr = sys.stderr
        self._original_stdin  = sys.stdin
        self._stderr_buffer   = StringIO()
        self._stdout_buffer   = StringIO()
        
        if argv is None:
            argv = sys.argv
        if parse:
            self.parseArgs(argv)


    def msg(self, msg, stdout=True):
        if stdout:
            sys.stdout.write(msg+'\n')
        else:
            sys.stderr.write(msg+'\n')
        if not self.test:
            sys.exit(1)


    def test_from_path(self, path):
        spath = path.split('::')
        if len(spath) == 3:
            return dict(
                    path        = spath[0],
                    class_name  = tokenizer.valid_class_name(spath[1]),
                    method_name = tokenizer.valid_method_name(spath[2])
                    )
        elif len(spath) == 2:
            return dict(
                    path       = spath[0],
                    class_name = tokenizer.valid_class_name(spath[1]),
                    )
        else:
            return dict(path = path)


    def path_from_argument(self, argv):
        # Get rid of the executable
        p_argv = argv[1:]
        valid_path = [path for path in p_argv if os.path.exists(os.path.abspath(path.split('::')[0]))]
        if valid_path:
            tests = self.test_from_path(os.path.abspath(valid_path[0]))
            return dict(
                    path        = tests.get('path'),
                    class_name  = tests.get('class_name'),
                    method_name = tests.get('method_name')
                    )
        else:
            return dict(path = os.getcwd())


    def capture(self):
        if self.config['capturing'] is True:
            sys.stderr = self._stderr_buffer
            sys.stdout = self._stdout_buffer
            sys.stdin  = DontReadFromInput()


    def end_capture(self):
        if self.config['capturing'] is True:
            sys.stderr = self._original_stderr
            sys.stdout = self._original_stdout
            sys.stdin  = self._original_stdin


    def parseArgs(self, argv):
        options = ['no-capture', '-s', 'fail', '-x', '-t', '-d',
                   'dots', 'traceback', 'tracebacks', 'describe', 'it']
        coverage_options  = ['--show-missing', '--cover-dir', '--cover-report',
                            'cover']
        profiling_options = ['-p', 'profile']
        options.extend(coverage_options)
        options.extend(profiling_options)

        args = ArgOpts(options)
        args.parse_args(argv)
        
        if args.catches_help():
            self.msg(self.konira_help)

        if args.catches_version():
            message = "konira version %s" % __version__
            self.msg(message)

        # Get a valid path
        path_info   = self.path_from_argument(argv)
        search_path = path_info.get('path')
        self.config['class_name']  = path_info.get('class_name')
        self.config['method_name'] = path_info.get('method_name')

        if args.match:

            # Matches for Describe
            if args.has('describe'):
                value = args.get_value('describe')
                if value:
                    self.config['class_name'] = tokenizer.valid_class_name(value)
                else:
                    self.msg("No valid 'describe' name")

            # Matches for it
            if args.has('it'):
                value = args.get_value('it')
                if value:
                    self.config['method_name'] = tokenizer.valid_method_name(value)
                else:
                    self.msg("No valid 'it' name")

            # Dotted output
            if args.has(['-d','dots']):
                self.config['dotted'] = True

            # Traceback options
            if args.has(['-t', 'traceback']):
                self.config['traceback'] = True

            # Fail options
            if args.has(['-x', 'fail']):
                self.config['first_fail'] = True

            # Capturing options
            if args.has(['-s', 'no-capture']):
                self.config['capturing'] = False

            # Profiling options
            if args.has(['-p', 'profile']):
                self.config['profiling'] = True
                self.config['dotted']    = True

            # Coverage options
            if args.has('cover'):
                self.running_coverage = True
                coverage_options = {}
                value = args.get_value('cover')
                if value:
                    coverage_options['coverpackages'] = [value]
                if args.has('--show-missing'):
                    coverage_options['show_missing'] = True

                run_cover = cover.DoCoverage(coverage_options)


        test_files = FileCollector(search_path)

        if not test_files:
            self.msg("No cases found to test.")
        try:
            self.capture()
            test_runner = Runner(test_files, self.config)
            test_runner.run()
            self.end_capture()
            report = ReportResults(test_runner)
            report.report()
            if self.running_coverage:
                run_cover.konira_terminal_summary()
            if test_runner.failures or test_runner.errors:
                sys.exit(2)
        except KeyboardInterrupt:
            self.msg("Exiting from konira.")






import konira
import inspect
from konira.exc         import (KoniraIOError, KoniraExecutionError, KoniraNoSkip, 
                                KoniraReassertError, KoniraFirstFail)
from konira             import exc



describe "dont read from input":


    before each:
        self.dont_read = exc.DontReadFromInput()


    it "asks if you are using pdb":
        message = "reading from stdin while output is captured (using pdb?)"
        assert self.dont_read.msg == message


    it "throws an exception if you call flush":
        raises KoniraIOError: self.dont_read.flush()


    it "throws an exception if you call flush with any arguments":
        raises KoniraIOError: self.dont_read.flush(True, False)


    it "throws an exception if you call write":
        raises KoniraIOError: self.dont_read.write()


    it "throws an exception if you call write with any arguments":
        raises KoniraIOError: self.dont_read.write(True, False)


    it "throws an exception if you call read":
        raises KoniraIOError: self.dont_read.read()


    it "throws an exception if you call read with any arguments":
        raises KoniraIOError: self.dont_read.read(True, False)


    it "raises when you call readline, readlines or iter":
        raises KoniraIOError: self.dont_read.__iter__()
        raises KoniraIOError: self.dont_read.readline()
        raises KoniraIOError: self.dont_read.readlines()



describe "konira execution error exception":


    before all:
        self.exc_err = exc.KoniraExecutionError 

    
    it "is an Exception type":
        type(self.exc_err) == Exception
    

    it "raises when there are insufficient args":
        raises TypeError: self.exc_err()


    it "raises a konira execution error":
        args = ('exc_name', 'filename', 1, 'a message', 'exc')
        exc = self.exc_err('a', 'b', 'c', 'd', 'e')
        raises KoniraExecutionError: raise exc



describe "konira no skip exception":


    it "raises a konira no skip exception":
        exc_err = exc.KoniraNoSkip
        raises KoniraNoSkip: raise exc_err



describe "konira reassert error exception":


    it "raises a reassert error exception":
        exc_err = exc.KoniraReassertError
        raises KoniraReassertError: raise exc_err



describe "konira first fail exception":


    it "raises a first fail exception":
        exc_err = exc.KoniraFirstFail
        raises KoniraFirstFail: raise exc_err



describe "konira IO Error exception":


    it "raises TypeError when arguments are not enough":
        exc_err = exc.KoniraIOError
        raises TypeError: raise exc_err

    it "raises a konira IO Error":
        exc_err = exc.KoniraIOError
        raises KoniraIOError: raise exc_err("an exception message")
    


describe "Source False value and assertions":

    before all:
        try:
            assert False
        except:
            self.false_trace = inspect.trace()[0]

    before each:
        self.source = exc.Source(self.false_trace)


    it "returns invalid when it cannot match correctly":
        assert self.source.is_valid == False


    it "can return the actual assert line stripping out assert":
        assert self.source.line == 'False'


    it "parses and evals a single assert statement":
        assert self.source._left_text == 'False'
        assert self.source.left_value == False


    it "raises index error when doing eval of a right value of a single assert":
        raises IndexError: self.source.right_value


    it "is able to get the actual source line":
        assert self.source.line == 'False'


    it "should not be valid because there is no operand":
        assert self.source.is_valid == False


describe "Source equality assertions and values":

    before all:
        try:
            assert 1 == 2
        except:
            self.eq_trace = inspect.trace()[0]

    before each:
        self.source = exc.Source(self.eq_trace)


    it "catches equality operand":
        assert self.source.operand == '=='


    it "parses and evalueates left values and text values with an equality operand":
        assert self.source._left_text == '1'
        assert self.source.left_value == 1


    it "parses and evaluates right values and text values with an equality operand":
        assert self.source._right_text == '2'
        assert self.source.right_value == 2


    it "is able to get the actual source line":
        assert self.source.line ==  '1 == 2'



describe "Source more than or equal assertions and values":


    before all:
        try:
            assert 1 >= 2
        except:
            self.mte_trace = inspect.trace()[0]


    before each:
        self.source = exc.Source(self.mte_trace)


    it "catches more than or equal operand":
        assert self.source.operand == '>='


    it "parses and evalueates left values and text values with an equality operand":
        assert self.source._left_text == '1'
        assert self.source.left_value == 1


    it "parses and evaluates right values and text values with an equality operand":
        assert self.source._right_text == '2'
        assert self.source.right_value == 2


    it "is able to get the actual source line":
        assert self.source.line == '1 >= 2'



describe "Source less than or equal assertions and values":


    before all:
        try:
            assert 2 <= 1
        except:
            self.lte_trace = inspect.trace()[0]


    before each:
        self.source = exc.Source(self.lte_trace)


    it "catches less than or equal operand":
        assert self.source.operand == '<='


    it "parses and evalueates left values and text values with an equality operand":
        assert self.source._left_text == '2'
        assert self.source.left_value == 2


    it "parses and evaluates right values and text values with an equality operand":
        assert self.source._right_text == '1'
        assert self.source.right_value == 1
     

    it "is able to get the actual source line":
        assert self.source.line == '2 <= 1'



describe "Source not equal assertions and values":


    before all:
        try:
            assert 'foo' != 'foo'
        except:
            self.ne_trace = inspect.trace()[0]


    before each:
        self.source = exc.Source(self.ne_trace)
        
        
    it "catches not equal operand":
        assert self.source.operand == '!='


    it "parses and evaluates right values and text values with a not equal operand":
        assert type(self.source._right_text) == str
        assert self.source.right_value       == 'foo'


    it "parses and evaluates left values and text values with a not equal operand":
        assert type(self.source._left_text) == str
        assert self.source.left_value       == 'foo'


    it "is able to get the actual source line":
        assert self.source.line == """'foo' != 'foo'"""

    

describe "Source is assertions and values":


    before all:
        try:
            assert 'foo' is 'Foo'
        except:
            self.is_trace = inspect.trace()[0]


    before each:
        self.source = exc.Source(self.is_trace)


    it "catches the is keyword operand":
        assert self.source.operand == ' is '


    it "parses and evals right values and text values with an is operand":
        assert type(self.source._right_text) == str
        assert self.source.right_value       == 'Foo'

    
    it "parses and evals left values and text values with an is operand":
        assert type(self.source._left_text) == str
        assert self.source.left_value       == 'foo'


    it "is able to get the actual source line":
        assert self.source.line == """'foo' is 'Foo'"""



describe "Source more than assertions and values":


    before all:
        try:
            assert 1 > 2
        except:
            self.mt_trace = inspect.trace()[0]

    before each:
        self.source = exc.Source(self.mt_trace)


    it "catches more than operand":
        assert self.source.operand == '>'


    it "parses and evaluates left values and text values with an equality operand":
        assert self.source._left_text == '1'
        assert self.source.left_value == 1


    it "parses and evaluates right values and text values with an equality operand":
        assert self.source._right_text == '2'
        assert self.source.right_value == 2


    it "is able to get the actual source line":
        assert self.source.line == '1 > 2'
    


describe "Source less than assertions and values":


    before all:
        try:
            assert 2 < 1
        except:
            self.lt_trace = inspect.trace()[0]


    before each:
        self.source = exc.Source(self.lt_trace)


    it "catches less than operand":
        assert self.source.operand == '<'


    it "parses and evals right values and text values with a less than operand":
        assert self.source._right_text == '1'
        assert self.source.right_value == 1


    it "parses and evals left values and text values with a less than operand":
        assert self.source._left_text == '2'
        assert self.source.left_value == 2


    it "is able to get the actual source line":
        assert self.source.line == '2 < 1'



describe "Source not in values and assertions":


    before all:
        try:
            assert 'foo' not in 'a foo here'
        except:
            self.notin_trace = inspect.trace()[0]


    before each:
        self.source = exc.Source(self.notin_trace)


    it "catches the not int keyword operand":
        assert self.source.operand == ' not in '


    it "parses and evals left and text values from a not in operand":
        assert type(self.source._left_text) == str
        assert self.source.left_value       == 'foo'


    it "parses and evals right and text values from a not in operand":
        assert type(self.source._right_text) == str
        assert self.source.right_value       == 'a foo here'


    it "is able to get the actual source line":
        assert self.source.line == """'foo' not in 'a foo here'"""



describe "konira assert helper function":


    before all:
        try:
            assert False
        except:
            self.trace = inspect.trace()[0]

        try:
            assert "long string" == "Long string"
        except:
            self.bad_trace = inspect.trace()[0]

        try:
            assert ('a',1) == ('b',1)
        except:
            self.tpl_trace = inspect.trace()[0]

        try:
            assert {'a':1} == {'b':1}
        except:
            self.dict_trace = inspect.trace()[0]


    it "is None when the source is invalid":
        reassert = exc.konira_assert(self.trace)
        assert reassert == None


    it "returns a valid diff when comparing strings":
        reassert = exc.konira_assert(self.bad_trace)
        description = ['long string == Long string', '- long string', '? ^', '+ Long string', '? ^']
        assert reassert == description
        
        
    it "returns valid dictionary comparisons":
        reassert = exc.konira_assert(self.dict_trace)
        description = ["{'a': 1} == {'b': 1}", "- {'a': 1}", '?   ^', "+ {'b': 1}", '?   ^']
        assert reassert == description


    it "returns tuple comparisons":
        reassert = exc.konira_assert(self.tpl_trace)
        description = ["('a', 1) == ('b', 1)", "At index 0 diff: 'a' != 'b'", "- ('a', 1)", '?   ^', "+ ('b', 1)", '?   ^']
        assert reassert == description



describe "Assert repr from source lines":


    it "returns None when it does not have an equality operand":
        _repr = exc.assertrepr_compare('==', None, '')
        assert _repr == None
